@@ -1,7 +1,20 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-'''Plotting vector data.'''
+'''Plotting vector data.
+
+This module can be used in two different ways:
+
+1. As a library. Just import the module and call the functions.
+   This is the way, how this module is used in openpivgui, for
+   example.
+
+2. As a terminal-application. Execute 
+   python3 -m openpivgui.vec_plot --help
+   for more information.
+   This is the way, how this module ist used in JPIV, for example.
+   For now, not all functions are callable in this way.
+'''
 
 __licence__ = '''
 This program is free software: you can redistribute it and/or modify
@@ -26,17 +39,70 @@ import pandas as pd
 import matplotlib.pyplot as plt
 from copy import copy
 from matplotlib.figure import Figure
+from matplotlib.colors import LinearSegmentedColormap
 
 
-def histogram(fname, figure, quantity, bins, log_y):
+# creating a custom rainbow colormap
+import matplotlib
+from matplotlib import pyplot as plt
+import numpy as np
+
+# creating a custom rainbow colormap
+short_rainbow = {'red':(
+                 (0.0, 0.0, 0.0),
+                 (0.2, 0.2, 0.2),
+                 (0.5, 0.0, 0.0),
+                 (0.8, 1.0, 1.0),
+                 (1.0, 1.0, 1.0)),
+        'green':((0.0, 0.0, 0.0),
+                 (0.2, 1.0, 1.0),
+                 (0.5, 1.0, 1.0),
+                 (0.8, 1.0, 1.0),
+                 (1.0, 0.0, 0.0)),
+        'blue': ((0.0, 1.0, 1.0),
+                 (0.2, 1.0, 1.0),
+                 (0.5, 0.0, 0.0),
+                 (0.8, 0.0, 0.0),
+                 (1.0, 0.0, 0.0))}
+
+long_rainbow = {'red': 
+                ((0.0, 0.0, 0.0),
+                 (0.1, 0.5, 0.5),
+                 (0.2, 0.0, 0.0),
+                 (0.3, 0.2, 0.2),
+                 (0.5, 0.0, 0.0),
+                 (0.7, 1.0, 1.0),
+                 (0.8, 1.0, 1.0),
+                 (1.0, 1.0, 1.0)),
+        'green':((0.0, 0.0, 0.0),
+                 (0.1, 0.0, 0.0),
+                 (0.2, 0.0, 0.0),
+                 (0.3, 1.0, 1.0),
+                 (0.5, 1.0, 1.0),
+                 (0.7, 1.0, 1.0),
+                 (0.8, 0.0, 0.0),
+                 (1.0, 0.3, 0.3)),
+        'blue': ((0.0, 0.0, 0.0),
+                 (0.1, 0.5, 0.5),
+                 (0.2, 1.0, 1.0),
+                 (0.3, 1.0, 1.0),
+                 (0.5, 0.0, 0.0),
+                 (0.7, 0.0, 0.0),
+                 (0.8, 0.0, 0.0),
+                 (1.0, 1.0, 1.0))}
+
+short_rainbow = LinearSegmentedColormap('my_colormap',short_rainbow,256)
+long_rainbow = LinearSegmentedColormap('my_colormap',long_rainbow,256)
+
+def histogram(data, figure, quantity, bins, log_y):
     '''Plot an histogram.
 
     Plots an histogram of the specified quantity.
 
     Parameters
     ----------
-    fname : str
-        A filename containing vector data.
+    data : pandas.DataFrame
+        Data to plot.
     figure : matplotlib.figure.Figure
         An (empty) Figure object.
     quantity : str
@@ -46,7 +112,7 @@ def histogram(fname, figure, quantity, bins, log_y):
     log_scale : boolean
         Use logaritmic vertical axis.
     '''
-    data = np.loadtxt(fname)
+
     if quantity == 'v':
         xlabel = 'absolute displacement'
         h_data = np.array([(l[2]**2+l[3]**2)**0.5 for l in data])
@@ -65,22 +131,26 @@ def histogram(fname, figure, quantity, bins, log_y):
     ax.set_title(parameter['plot_title'])
 
 
-def profiles(data, figure, orientation):
+def profiles(data, parameter, fname, figure, orientation):
     '''Plot velocity profiles.
 
     Line plots of the velocity component specified.
 
     Parameters
     ----------
+    data : pandas.DataFrame
+        Data to plot.
     fname : str
-        A filename containing vector data.
+        A filename containing vector data. 
+        (will be deprecated in later updates)
     figure : matplotlib.figure.Figure 
         An (empty) Figure object.
     orientation : str 
         horizontal: Plot v_y over x.
         vertical: Plot v_x over y.
     '''
-    data = data.to_numpy().astype(np.float)
+    #data = data.to_numpy().astype(np.float)
+    data = np.loadtxt(fname)
     
     dim_x, dim_y = get_dim(data)
     
@@ -92,7 +162,7 @@ def profiles(data, figure, orientation):
         xlabel = 'x position'
         ylabel = 'y displacement'
         
-        for i in range(dim_y):
+        for i in range(0, dim_y, parameter['profiles_jump']):
             p_data.append(data[dim_x*i:dim_x*(i+1),3])
         #print(p_data[-1])
         for p in p_data:
@@ -103,7 +173,7 @@ def profiles(data, figure, orientation):
         xlabel = 'y position'
         ylabel = 'x displacement'
         
-        for i in range(dim_x):
+        for i in range(0, dim_x, parameter['profiles_jump']):
             p_data.append(data[i::dim_x,2])
             
         for p in p_data:
@@ -121,8 +191,8 @@ def scatter(data, figure):
 
     Parameters
     ----------
-    fname : str
-        Name of a file containing vector data.
+    data : pandas.DataFrame
+        Data to plot.
     figure : matplotlib.figure.Figure 
         An (empty) Figure object.
     '''
@@ -145,8 +215,8 @@ def vector(data, parameter, figure, invert_yaxis=True, valid_color='blue',
 
     Parameters
     ----------
-    fname : str
-        Pathname of a text file containing vector data.
+    data : pandas.DataFrame
+        Data to plot.
     figure : matplotlib.figure.Figure 
         An (empty) Figure object.
     '''
@@ -201,9 +271,13 @@ def contour(data, parameter, figure):
     # iteration to set value types to float
     for i in list(data.columns.values):
         data[i] = data[i].astype(float)
-    # calculate absolute velocity and add it in a new column in data    
-    data['abs'] = (data.vx**2 + data.vy**2)**0.5
-    
+    # choosing velocity for the colormap and add it to an new colummn in data
+    if parameter['velocity_color'] == 'vx':
+        data['abs'] = data.vx
+    elif parameter['velocity_color'] == 'vy':
+        data['abs'] = data.vy
+    else:
+        data['abs'] = (data.vx**2+data.vy**2)**0.5
     # pivot table for contour function    
     data_pivot = data.pivot(index = 'y',
                             columns = 'x',
@@ -228,28 +302,154 @@ def contour(data, parameter, figure):
     else:
         levels = int(parameter['color_levels'])
     # Choosing the correct colormap
-    if parameter['color_map'] == 'None':
-        colormap = None
+    if parameter['color_map'] == 'short rainbow':
+        colormap = short_rainbow
+    elif parameter['color_map'] == 'long rainbow':
+        colormap = long_rainbow
     else:
         colormap = parameter['color_map']
     # set contour plot to the variable fig to add a colorbar 
+    if parameter['extend_cbar']:
+        extend = 'both'
+    else:
+        extend = None
     fig = ax.contourf(data_pivot.columns, 
                 data_pivot.index, 
                 data_pivot.values, 
                 levels = levels, 
                 cmap = colormap,
                 vmin = vmin,
-                vmax = vmax)
+                vmax = vmax,
+                extend = extend)
+    
     # set the colorbar to the variable cb to add a description
     cb = plt.colorbar(fig, ax=ax)
+    
+    # set origin to top left or bottom left
+    if parameter['invert_yaxis']:
+        ax.set_ylim(ax.get_ylim()[::-1])
+        
     # description to the contour lines
     cb.ax.set_ylabel('Velocity [m/s]')
+    
     # labels for the axes
     ax.set_xlabel('x-position')
     ax.set_ylabel('y-position')
+    
     # plot title from the GUI
     ax.set_title(parameter['plot_title'])
     
+def contour_and_vector(data, parameter, figure, **kw):
+    '''Display a contour plot    
+
+    Parameters
+    ----------
+    data : pandas.DataFrame
+        Data to plot.
+    parameter : openpivgui.OpenPivParams
+        Parameter-object.
+    figure : matplotlib.figure.Figure
+       An (empty) Figure object.
+    '''
+    # figure for subplot
+    ax = figure.add_subplot(111)
+    # iteration to set value types to float
+    for i in list(data.columns.values):
+        data[i] = data[i].astype(float)
+    # choosing velocity for the colormap and add it to an new colummn in data        
+    if parameter['velocity_color'] == 'vx':
+        data['abs'] = data.vx
+    elif parameter['velocity_color'] == 'vy':
+        data['abs'] = data.vy
+    else:
+        data['abs'] = (data.vx**2+data.vy**2)**0.5
+    # pivot table for contour function    
+    data_pivot = data.pivot(index = 'y',
+                            columns = 'x',
+                            values = 'abs')
+    # try to get limits, if not possible set to None
+    try:
+        vmin = float(parameter['vmin'])
+    except:
+        vmin = None
+    try:
+        vmax = float(parameter['vmax'])
+    except:
+        vmax = None
+    # settings for color scheme of the contour plot  
+    if vmax is not None and vmin is not None:
+        levels = np.linspace(vmin, vmax, int(parameter['color_levels']))
+    elif vmax is not None:
+        levels = np.linspace(0, vmax, int(parameter['color_levels']))
+    elif vmin is not None:
+        vmax = data_pivot.max().max()
+        levels = np.linspace(vmin, vmax, int(parameter['color_levels']))
+    else:
+        levels = int(parameter['color_levels'])
+    # Choosing the correct colormap
+    if parameter['color_map'] == 'short rainbow':
+        colormap = short_rainbow
+    elif parameter['color_map'] == 'long rainbow':
+        colormap = long_rainbow
+    else:
+        colormap = parameter['color_map']
+
+    # set contour plot to the variable fig to add a colorbar 
+    if parameter['extend_cbar']:
+        extend = 'both'
+    else:
+        extend = None
+    fig = ax.contourf(data_pivot.columns, 
+                data_pivot.index, 
+                data_pivot.values, 
+                levels = levels, 
+                cmap = colormap,
+                vmin = vmin,
+                vmax = vmax,
+                extend = extend)
+    
+    # quiver plot
+    data = data.to_numpy().astype(np.float)
+
+    try:
+        invalid = data[:, 4].astype('bool')
+    except:
+        invalid = np.asarray([True for i in range(len(data))])
+    
+    # tilde means invert:
+    valid = ~invalid
+            
+    ax.quiver(data[invalid, 0],
+              data[invalid, 1],
+              data[invalid, 2],
+              data[invalid, 3],
+              color = parameter['invalid_color'],
+              label = 'invalid', **kw)
+    
+    ax.quiver(data[valid, 0],
+              data[valid, 1],
+              data[valid, 2],
+              data[valid, 3],
+              color = parameter['valid_color'],
+              label = 'valid', **kw)
+    
+    # set the colorbar to the variable cb to add a description
+    cb = plt.colorbar(fig, ax=ax)
+    
+    # set origin to top left or bottom left
+    if parameter['invert_yaxis']:
+        ax.set_ylim(ax.get_ylim()[::-1])
+        
+    # description to the contour lines
+    cb.ax.set_ylabel('Velocity [m/s]')
+    
+    # labels for the axes
+    ax.set_xlabel('x-position')
+    ax.set_ylabel('y-position')
+    
+    # plot title from the GUI
+    ax.set_title(parameter['plot_title'])    
+    
 def streamlines(data, parameter, figure):
     '''Display a streamline plot.    
 
@@ -276,8 +476,10 @@ def streamlines(data, parameter, figure):
         density = float(parameter['streamline_density'])
         
     # Choosing the correct colormap
-    if parameter['color_map'] == 'None':
-        colormap = None
+    if parameter['color_map'] == 'short rainbow':
+        colormap = short_rainbow
+    elif parameter['color_map'] == 'long rainbow':
+        colormap = long_rainbow
     else:
         colormap = parameter['color_map']
         
@@ -290,9 +492,9 @@ def streamlines(data, parameter, figure):
                          values = 'vy')
 
     # choosing data for the colormap
-    if parameter['streamlines_color'] == 'vx':
+    if parameter['velocity_color'] == 'vx':
         color_values = data_vx.values
-    elif parameter['streamlines_color'] == 'vy':
+    elif parameter['velocity_color'] == 'vy':
         color_values = data_vy.values
     else:
         color_values = (data_vx.values**2+data_vy.values**2)**0.5
@@ -359,9 +561,9 @@ def streamlines(data, parameter, figure):
         data_vy = data.pivot(index='y', columns='x', values='vy')
         
         # choosing data for the colormap
-        if parameter['streamlines_color'] == 'vx':
+        if parameter['velocity_color'] == 'vx':
             color_values = data_vx.values
-        elif parameter['streamlines_color'] == 'vy':
+        elif parameter['velocity_color'] == 'vy':
             color_values = data_vy.values
         else:
             color_values = (data_vx.values**2+data_vy.values**2)**0.5
@@ -380,6 +582,10 @@ def streamlines(data, parameter, figure):
     cb = plt.colorbar(fig.lines, ax=ax)
     cb.ax.set_ylabel('Velocity [m/s]')
     
+    # set origin to top left or bottom left
+    if parameter['invert_yaxis']:
+        ax.set_ylim(ax.get_ylim()[::-1])
+        
     # add diagram options    
     ax.set_xlabel('x-position')
     ax.set_ylabel('y-position')
@@ -476,11 +682,12 @@ def get_dim(array):
     '''Computes dimension of vector data.
 
     Assumes data to be organised as follows (example):
-    x  y  v_x v_y
-    16 16 4.5 3.2
-    32 16 4.3 3.1
-    16 32 4.2 3.5
-    32 32 4.5 3.2
+    x  y  v_x v_y ..
+    16 16 4.5 3.2 ..
+    32 16 4.3 3.1 ..
+    16 32 4.2 3.5 ..
+    32 32 4.5 3.2 ..
+    .. .. ..  ..
 
     Parameters
     ----------
@@ -495,7 +702,8 @@ def get_dim(array):
     return(len(set(array[:, 0])),
            len(set(array[:, 1])))
 
-                      
+
+
 if __name__=="__main__":
     parser = argparse.ArgumentParser(description='Plot vector data.')
     parser.add_argument('--plot_type',
@@ -504,7 +712,10 @@ if __name__=="__main__":
                         choices=['histogram',
                                  'profiles',
                                  'vector',
-                                 'scatter'],
+                                 'scatter',
+                                 'contour'
+                                 'contour_and_vector',
+                                 'streamlines'],
                         default='vector',
                         help='type of plot')
     parser.add_argument('--fname',
@@ -540,21 +751,29 @@ if __name__=="__main__":
                         default=True,
                         help='Invert y-axis of vector plot')
     args = parser.parse_args()
+    data = np.loadtxt(args.fname)
     fig = Figure()
     if args.plot_type=='histogram':
-        histogram(args.fname,
+        histogram(data,
                   fig,
                   quantity=args.quantity,
                   bins=args.bins,
                   log_y=args.log_y)
     elif args.plot_type=='profiles':
-        profiles(args.fname,
+        profiles(data,
                  fig,
                  orientation=args.orientation)
     elif args.plot_type=='vector':
-        vector(args.fname,
+        vector(data,
                fig,
                invert_yaxis=args.invert_yaxis)
     elif args.plot_type=='scatter':
-        scatter(args.fname, fig)
+        scatter(data,
+                fig)
+    elif args.plot_type=='contour':
+        print('Not yet implemented')
+    elif args.plot_type=='contour_and_vector':
+         print('Not yet implemented')
+    elif args.plot_type=='streamlines':
+         print('Not yet implemented')
     plt.show()
